'
' ####################
' #####  PROLOG  #####
' ####################

' The IDEA encryption algorithm. From wikipedia:

' http://en.wikipedia.org/wiki/International_Data_Encryption_Algorithm

' * In cryptography, the International Data Encryption Algorithm (IDEA) 
' * is a block cipher designed by Xuejia Lai (???) and James Massey of ETH Zurich 
' * and was first described in 1991. The algorithm was intended as a replacement 
' * for the Data Encryption Standard. IDEA is a minor revision of an earlier cipher, 
' * PES (Proposed Encryption Standard); IDEA was originally called IPES (Improved PES).

' * The cipher was designed under a research contract with the Hasler Foundation, 
' * which became part of Ascom-Tech AG. The cipher is patented in a number of 
' * countries but is freely available for non-commercial use. The name "IDEA" is 
' * also a trademark. The patents will expire in 20102011. Today, IDEA is licensed 
' * worldwide by MediaCrypt.

' * IDEA was used in Pretty Good Privacy (PGP) V2.0, and was incorporated after 
' * the original cipher used in v1.0, BassOmatic, was found to be insecure. 
' * IDEA is an optional algorithm in OpenPGP.

' * IDEA is registered as the international patent WO 91/18459
' * "Device for Converting a Digital Block and the Use thereof".
' * For commercial use of IDEA, you should contact:
' * ASCOM TECH AG
' * Freiburgstrasse 370
' * CH-3018 Bern, Switzerland

VERSION "0.0001"
CONSOLE

TYPE dbyte = USHORT
TYPE qbyte = ULONG
TYPE word = USHORT
TYPE dword = ULONG
TYPE long = SLONG

TYPE IDEA_USERKEY
	UNION
		STRING * 16 .userkey
		USHORT .ukey[7]
	END UNION
END TYPE

TYPE IDEA_BUFFER
	UNION
		STRING * 8 .buffer
		USHORT .buf[3]
	END UNION
END TYPE
'
	IMPORT "xst"				' Standard library : required by most programs
	IMPORT "xsx"				' Extended standard library
'	IMPORT "xio"				' Console input/ouput library
'	IMPORT "xst_s.lib"
' IMPORT "xsx_s.lib"
' IMPORT "xio_s.lib"

	IMPORT "msvcrt"		  ' msvcrt.dll
'	IMPORT "gdi32"
'	IMPORT "user32"
	IMPORT "kernel32"

$$mulMod             = 0x10001     '  2**16 + 1                       
$$addMod             = 0x10000     '  2**16                           
$$ones               = 0xFFFF      '  2**16 - 1                       

$$nofKeyPerRound     = 6           '  Number of used keys per round    
$$nofRound           = 8           '  Number of rounds                 

$$dataSize           = 8           '  8 bytes = 64 bits                
$$dataLen            = 4
$$keySize            = 104         '  104 bytes = 832 bits             
$$keyLen             = 52
$$userKeySize        = 16          '  16 bytes = 128 bits             
$$userKeyLen         = 8
$$kbytes        		 = 4096

DECLARE FUNCTION Entry ()
DECLARE FUNCTION idea (dbyte dataIn[], dbyte dataOut[], dbyte key[])
DECLARE FUNCTION invert_idea_key (dbyte key[], dbyte invKey[])
DECLARE FUNCTION qbyte Mul (qbyte a, qbyte b)
DECLARE FUNCTION dbyte MulInv (dbyte x)
DECLARE FUNCTION expand_user_key (dbyte userKey[], dbyte key[])
DECLARE FUNCTION idea_crypt_data (dataIn$, dataOut$, key$, encrypt)

FUNCTION Entry ()

	dbyte userkey[$$userKeyLen - 1]
	dbyte key[$$keyLen - 1]
	dbyte dekey[$$keyLen - 1]
	dbyte in[$$dataLen - 1]
	dbyte out[$$dataLen - 1]
	dbyte out2[$$dataLen - 1]
	
	' make a sample user key
  FOR i = 0 TO 7
    userkey[i] = i+1
	NEXT i
	
	' expand user key 
	expand_user_key (@userkey[], @key[])
	
	' create decryption key
	invert_idea_key (@key[], @dekey[])
	
  ' make a sample plain text pattern
  FOR i = 0 TO 3
    in[i] = i
	NEXT i
	
	printf (&"\nEncrypting %d kb (%ld blocks): ", $$kbytes, $$kbytes*64)
	start = clock()
	
	' encrypt data
	idea (@in[], @out[], @key[])
	upp = 64*$$kbytes - 1
  FOR i = 1 TO upp 
    idea (@out[], @out[], @key[])   				' repeated encryption 
	NEXT i
	
	' decrypt data
	idea (@out[], @out2[], @dekey[])
	FOR i = 1 TO upp
    idea (@out2[], @out2[], @dekey[])  			' repeated decryption
	NEXT i
	
  end = clock() - start
  l = end * 1000. / DOUBLE($$CLOCKS_PER_SEC + 1)
  i = l/1000.
  j = l MOD 1000
	l = $$kbytes * 1024. * $$CLOCKS_PER_SEC / end
  printf (&"%d.%03d seconds = %ld bytes/s\n", i, j, l)
	
	' check if in[] = out2[]
	printf (&"\nin   %6u   %6u  %6u  %6u \n", in[0], in[1], in[2], in[3])
  printf (  &"out  %6u   %6u  %6u  %6u \n", out[0], out[1], out[2], out[3])
  printf (  &"out2 %6u   %6u  %6u  %6u \n", out2[0], out2[1], out2[2], out2[3])
	
  ' Now decrypted out2[] should be same as original in[]
	FOR k = 0 TO 3
    IF (in[k] != out2[k]) THEN
      error = $$TRUE
		END IF
	NEXT k
	
	IF error THEN
		printf (&"\nError! Decryption doesn't match.\n")
	ELSE
		printf (&"\nDecryption ok.\n")
	END IF
	
	' test encryption of string
	PRINT
	PRINT "Encrypting a string"
	dataIn$ = "1234567890-=qwertyuiop[]asdfghjkl;'zxcvbnm,.?:}{+_)(*&^%$#@!~`"
	PRINT "dataIn$   "; dataIn$
	key$ = "passwordkeypro"
	idea_crypt_data (@dataIn$, @dataOut$, @key$, $$TRUE)
	PRINT "dataOut$  "; dataOut$
	
	' decrypt string
	idea_crypt_data (@dataOut$, @dataOut2$, @key$, $$FALSE)
	PRINT "dataOut2$ "; dataOut2$
	
	IF dataIn$ != dataOut2$ THEN 
		PRINT "Error! String encryption failed!"
	ELSE
		PRINT "Decryption ok."
	END IF
	
	' test encryption of a file
	PRINT
	PRINT "Encrypting a file"
	
	XstGetEnvironmentVariable ("XBLDIR", @dir$)
	file$ = dir$ + "/src/xblite/xblite.x"
	XstLoadString (@file$, @dataIn$)
	IF dataIn$ THEN
		PRINT "Filesize is"; LEN(dataIn$); " bytes."
		key$ = "cryptfilepassword"
		idea_crypt_data (@dataIn$, @dataOut$, @key$, $$TRUE)
		fo$ = "c:/temp/crypttest.dat"
		XstSaveString (fo$, @dataOut$)
		dataOut$ = ""
		XstLoadString (@fo$, @dataOut$)
		start = GetTickCount ()
		idea_crypt_data (@dataOut$, @dataOut2$, @key$, $$FALSE)
		fin = GetTickCount() - start
		PRINT "Encrypted in "; fin; " ms ="; ULONG((LEN(dataIn$)/(DOUBLE(fin)))*1000.); " bytes/s" 
	'	fo1$ = "c:/temp/crypttest.x"
	'	XstSaveString (fo1$, @dataOut2$)
		XstDeleteFile (@fo$)
		IF dataIn$ != dataOut2$ THEN 
			PRINT "Error! File encryption failed!"
		ELSE
			PRINT "Decryption ok."
		END IF
	ELSE
		PRINT "Error! File not found."
	END IF


  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION

FUNCTION idea (dbyte dataIn[], dbyte dataOut[], dbyte key[])

	qbyte round, x0, x1, x2, x3, t0, t1

	x0 = dataIn[0]
	x1 = dataIn[1]
	x2 = dataIn[2]
	x3 = dataIn[3]

	idx = 0
	FOR round = $$nofRound TO 1 STEP -1

		x0  = Mul (x0,   (key[idx])) 														: INC idx
		x1  =     (x1 +  (key[idx])) & $$ones 									: INC idx
		x2  =     (x2 +  (key[idx])) & $$ones 									: INC idx
		x3  = Mul (x3,   (key[idx])) 														: INC idx
		t0  = Mul (      (key[idx]), x0 ^ x2) 									: INC idx
		t1  = Mul (      (key[idx]), (t0 + (x1 ^ x3)) & $$ones) : INC idx
		t0  = (t0 + t1) & $$ones
		x0  = x0 ^ t1
		x3  = x3 ^ t0
		t0  = t0 ^ x1
		x1  = x2 ^ t1
		x2  = t0
	NEXT
	
	dataOut[0] = (Mul (x0,   (key[idx]))) 					: INC idx
	dataOut[1] = (    (x2 +  (key[idx])) & $$ones) 	: INC idx
	dataOut[2] = (    (x1 +  (key[idx])) & $$ones) 	: INC idx
	dataOut[3] = (Mul (x3,   (key[idx])))

END FUNCTION

FUNCTION invert_idea_key (dbyte key[], dbyte invKey[])

  dbyte dk[$$keyLen-1]
	
	idx = 0

	dk[$$nofKeyPerRound * $$nofRound + 0] = MulInv (key[idx]) 							: INC idx
	dk[$$nofKeyPerRound * $$nofRound + 1] = ($$addMod - key[idx]) & $$ones 	: INC idx
	dk[$$nofKeyPerRound * $$nofRound + 2] = ($$addMod - key[idx]) & $$ones 	: INC idx
	dk[$$nofKeyPerRound * $$nofRound + 3] = MulInv (key[idx]) 							: INC idx
		
	FOR i = $$nofKeyPerRound * ($$nofRound - 1) TO 0 STEP -$$nofKeyPerRound 
		dk[i + 4] =         key[idx]									: INC idx
		dk[i + 5] =         key[idx]									: INC idx
		dk[i + 0] = MulInv (key[idx])									: INC idx
		IF (i > 0) THEN
			dk[i + 2] = ($$addMod - key[idx]) & $$ones	: INC idx
			dk[i + 1] = ($$addMod - key[idx]) & $$ones	: INC idx
		ELSE
			dk[i + 1] = ($$addMod - key[idx]) & $$ones	: INC idx
			dk[i + 2] = ($$addMod - key[idx]) & $$ones	: INC idx
		END IF
		dk [i + 3] = MulInv (key[idx])								: INC idx
	NEXT i

	REDIM invKey[$$keyLen - 1]
  FOR i = 0 TO $$keyLen - 1 
    invKey[i] = dk[i]
	NEXT i

END FUNCTION

FUNCTION qbyte Mul (qbyte a, qbyte b)

	long p
	qbyte q

	IF (a == 0) THEN
		p = $$mulMod - b
	ELSE
		IF (b == 0) THEN
			p = $$mulMod - a
		ELSE
			q = a * b
			p = (q & $$ones) - (q >> 16)
			IF (p <= 0) THEN
				p = p + $$mulMod
			END IF
		END IF
	END IF
	RETURN (p & $$ones)

END FUNCTION

FUNCTION dbyte MulInv (dbyte x)

    long n1, n2, q, r, b1, b2, t

    IF (x == 0) THEN RETURN (0)

    n1 = $$mulMod
    n2 = x
    b2 = 1
    b1 = 0
    DO
			r = (n1 MOD n2)
			q = (n1 - r) / n2
			IF (r == 0) THEN
				IF (b2 < 0) THEN
					b2 = $$mulMod + b2
				END IF
			ELSE
				n1 = n2
				n2 = r
				t  = b2
				b2 = b1 - q * b2
				b1 = t
			END IF
    LOOP WHILE (r != 0)

    RETURN (b2)

END FUNCTION

FUNCTION expand_user_key (dbyte userKey[], dbyte key[])

	REDIM key[$$keyLen - 1]

	FOR index = 0 TO $$userKeyLen - 1 
		key[index] = userKey[index]
	NEXT index
			
	'  Shifts                                                   
	FOR index = $$userKeyLen TO $$keyLen - 1  
		IF ((index + 2) MOD 8 == 0) THEN       		'  For key [14], key [22], ..  
				key[index] = ((key[index - 7] & 127) << 9) ^ (key[index - 14]  >> 7)
		ELSE
			IF ((index + 1) MOD 8 == 0) THEN       	'  For key [15], key [23], ..
				key[index] = ((key[index - 15] & 127) << 9) ^ (key[index - 14] >> 7)
			ELSE
				key[index] = ((key[index - 7]  & 127) << 9) ^ (key[index - 6]  >> 7)
			END IF
		END IF
	NEXT index

END FUNCTION

FUNCTION idea_crypt_data (dataIn$, dataOut$, key$, encrypt)

' dataIn$  - string to encrypt/decrypt
' dataOut$ - returned encrypted/decrypted string
' key$     - user defined key string, maximum of 16 bytes
' encrypt  - $$TRUE = encrypt, $$FALSE = decrypt

	dbyte blockIn[3]
	dbyte blockOut[3]
	dbyte expkey[], invkey[]
	dbyte userkey[7]
	IDEA_USERKEY iuk
	IDEA_BUFFER ibuf

	IFZ dataIn$ THEN RETURN ($$TRUE)
	IFZ key$ THEN RETURN ($$TRUE)

	' align dataIn$ to 8 bytes
	crypt_block_size = 8
	buffer_size = LEN(dataIn$)
	align = ((buffer_size + 7) AND -8) - buffer_size
	IF align THEN
		dataIn$ = dataIn$ + NULL$(align)
		buffer_size = LEN(dataIn$)
	END IF
	
	dataOut$ = NULL$(buffer_size)
	pDataOut = &dataOut$
	pDataIn  = &dataIn$
	
	nbr_blocks = buffer_size / crypt_block_size

	' blocks must be whole number (8 byte alignment)
  IF ((nbr_blocks * crypt_block_size) != buffer_size) THEN
    RETURN ($$TRUE)
	END IF

	' key$ cannot be longer than 16 bytes
	IF LEN(key$) > 16 THEN
		key$ = LEFT$(key$, 16)
	END IF
	iuk.userkey = key$

	' convert key$ to a USHORT array userkey[]
	FOR i = 0 TO $$userKeyLen-1
		userkey[i] = iuk.ukey[i]
	NEXT i 

	' create expanded user key array
	expand_user_key (@userkey[], @expkey[])
	
	' invert the key for decryption 
	IFZ encrypt THEN
		invert_idea_key (@expkey[], @invkey[])
		
		FOR index = 0 TO buffer_size - 1 STEP 8
			memcpy (&ibuf.buffer, pDataIn + index, 8)
			FOR j = 0 TO 3 
				blockIn[j] = ibuf.buf[j]
			NEXT j

			' decrypt data block
			idea (@blockIn[], @blockOut[], @invkey[])
			
			FOR j = 0 TO 3
				ibuf.buf[j] = blockOut[j]
			NEXT j
			memcpy (pDataOut + index, &ibuf.buffer, 8)
		NEXT index
	ELSE
		FOR index = 0 TO buffer_size - 1 STEP 8
			memcpy (&ibuf.buffer, pDataIn + index, 8)
			FOR j = 0 TO 3 
				blockIn[j] = ibuf.buf[j]
			NEXT j
			' encrypt data block
			idea (@blockIn[], @blockOut[], @expkey[])
			
			FOR j = 0 TO 3
				ibuf.buf[j] = blockOut[j]
			NEXT j
			memcpy (pDataOut + index, &ibuf.buffer, 8)
		NEXT index
	END IF

	END FUNCTION
END PROGRAM